@@ -421,8 +421,16 @@ XPoint UGetCursor()
    u_int   nullui;
    XPoint newpos;
  
+#ifdef TCL_WRAPPER
+   /* Don't use areastruct.areawin;  if called from inside an object	*/
+   /* (e.g., "here" in a Tcl expression), areastruct.areawin will be	*/
+   /* an off-screen pixmap, and cause a crash.				*/
+   XQueryPointer(dpy, Tk_WindowId(areastruct.area), &nullwin, &nullwin,
+	&nullint, &nullint, &xpos, &ypos, &nullui);
+#else
    XQueryPointer(dpy, areastruct.areawin, &nullwin, &nullwin, &nullint,
 	&nullint, &xpos, &ypos, &nullui);
+#endif
 
    newpos.x = xpos;
    newpos.y = ypos;
@@ -1829,6 +1837,7 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
    short	tmpwidth;
    int		defaultcolor = passcolor;
    int		curcolor = passcolor;
+   int		thispart;
    XPoint 	bboxin, bboxout[2];
    u_char	xm, ym;
    objectptr	theobject = theinstance->thisobject;
@@ -1866,9 +1875,11 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
      XSetLineAttributes(dpy, areastruct.gc, tmpwidth >= 2.0 ? (int)tmpwidth :
 		0, LineSolid, CapRound, JoinBevel);
 
-     for (areagen = theobject->plist; areagen < theobject->plist +
-           theobject->parts; areagen++) {
+     /* guard against plist being regenerated during a redraw by the	*/
+     /* expression parameter mechanism (should that be prohibited?)	*/
 
+     for (thispart = 0; thispart < theobject->parts; thispart++) {
+       areagen = theobject->plist + thispart;
        if ((*areagen)->type & DRAW_HIDE) continue;
 
        if (defaultcolor != DOFORALL) {
